/*****************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No 
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all 
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM 
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES 
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS 
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of 
* this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2014-2016 Renesas Electronics Corporation. All rights reserved.
******************************************************************************/
/*****************************************************************************
* File Name    : r_dac_rx.c
* Description  : Functions for using DAC on RX devices.
******************************************************************************
* History : DD.MM.YYYY Version Description
*           31.03.2014 2.00    New API
*           05.09.2014 2.10    Added support for RX64M.
*           20.01.2015 2.20    Added support for RX113.
*           02.03.2015 2.30    Added support for RX71M.
*           10.06.2015 2.40    Added support for RX231.
*           30.09.2015 2.50    Added support for RX23T.
*           01.10.2015 2.60    Added support for RX130.
*           01.12.2015 2.70    Added support for RX230.
*           01.02.2016 2.80    Added support for RX24T.
*           31.08.2016 2.91    Added support for RX65N.
*           19.12.2016 3.00    Added support for RX24U.
*                              Corrected so that AMADSEL bit does not become "1",
*                               when synchronous conversion is enabled. (RX64M/RX71M/RX65N)
******************************************************************************/
/*****************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "platform.h"
#include "r_dac_rx_if.h"
#include "r_dac_rx_config.h"
/*****************************************************************************
Typedef definitions
******************************************************************************/

/*****************************************************************************
Macro definitions
******************************************************************************/
#define DAC_CHANS_OFF   (0x1F)
#if (DAC_CFG_NUM_CH == 1)
#define DAC_CHANS_ON    (0xDF)
#else
#define DAC_CHANS_ON    (0x5F)
#endif

/*****************************************************************************
Private global variables and functions
******************************************************************************/
static void power_on(void);
static void power_off(void);
static dac_err_t dac_set_options(dac_cfg_t *p_cfg);

/*****************************************************************************
* Function Name: R_DAC_Open
* Description  : Initializes the DAC peripheral
* Arguments    : p_cfg -
*                     pointer to configuration structure
* Return Value : DAC_SUCCESS -
*                    DAC initialized successfully
*                DAC_ERR_NULL_PTR
*                    p_cfg is NULL
*                DAC_ERR_INVALID_ARG
*                    illegal unit number (RX64M, RX71M, RX65N, RX651)
*                DAC_ERR_LOCK_FAILED -
*                    DAC already opened
*                DAC_ERR_ADC_NOT_POWERED -
*                    Cannot sync because ADC is not powered
*                DAC_ERR_ADC_CONVERTING
*                    Cannot sync with ADC because it is doing a conversion
******************************************************************************/
dac_err_t R_DAC_Open(dac_cfg_t *p_cfg)
{
dac_err_t err;

#if (DAC_CFG_PARAM_CHECKING_ENABLE == 1)
    if ((p_cfg == NULL) || (p_cfg == FIT_NO_PTR))
    {
        return DAC_ERR_NULL_PTR;
    }
#if defined(BSP_MCU_RX64M) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N)
    if (p_cfg->sync_unit > 1)
    {
        return DAC_ERR_INVALID_ARG;
    }
#endif
#endif

    if (R_BSP_HardwareLock(BSP_LOCK_DA) == false)
    {
        return DAC_ERR_LOCK_FAILED;
    }

    power_on();

    /* Set registers to default state */
    DA.DACR.BYTE = DAC_CHANS_OFF;       // channel output disabled
    DA.DADR0 = 0;                       // ch0 data = 0
#if (DAC_CFG_NUM_CH > 1)
    DA.DADR1 = 0;                       // ch1 data = 0;
#endif

    /* Set options */
    err = dac_set_options(p_cfg);
    if (err != DAC_SUCCESS)
    {
        R_BSP_HardwareUnlock(BSP_LOCK_DA);
    }

    return err;
}


/*****************************************************************************
* Function Name: dac_set_options
* Description  : Configures MCU-unique options.
*                Note: See Application Note for this module for pin configuration requirements.
* Arguments    : p_cfg -
*                     pointer to configuration structure
* Return Value : DAC_SUCCESS -
*                    DAC configured successfully
*                DAC_ERR_INVALID_ARG -
*                    Could not set reference voltage
*                DAC_ERR_ADC_NOT_POWERED
*                    Cannot sync because ADC is not powered
*                DAC_ERR_ADC_CONVERTING
*                    Cannot sync because ADC is converting
******************************************************************************/
static dac_err_t dac_set_options(dac_cfg_t *p_cfg)
{
    dac_err_t err = DAC_SUCCESS;

    /* OPTION: SELECT DAC VOLTAGE REFERENCE */

#if defined(BSP_MCU_RX113) || defined(BSP_MCU_RX231) || defined(BSP_MCU_RX230)
#if (DAC_CFG_PARAM_CHECKING_ENABLE == 1)
    /* CHECK PARAMETERS */
    if ((p_cfg->ref_voltage != DAC_REFV_AVCC0_AVSS0) && (p_cfg->ref_voltage != DAC_REFV_INTERNAL_AVSS0)
            && (p_cfg->ref_voltage != DAC_REFV_VREFH_VREFL))
    {
        return DAC_ERR_INVALID_ARG;
    }
#endif

    DA.DAVREFCR.BYTE = p_cfg->ref_voltage;
#endif

    /* OPTION: FORMAT */
    DA.DADPR.BIT.DPSEL = (uint8_t)((p_cfg->fmt_flush_right == true) ? 0 : 1);


    /* OPTION: SYNCHRONIZE WITH ADC */

#if defined(BSP_MCU_RX113) || defined(BSP_MCU_RX130) || defined(BSP_MCU_RX231) || defined(BSP_MCU_RX230) || defined(BSP_MCU_RX63_ALL)
    if (p_cfg->sync_with_adc == false)
    {
        DA.DAADSCR.BIT.DAADST = 0;      // do not sync with ADC
    }
    else if ((SYSTEM.MSTPCRA.BIT.MSTPA17 == 0) && (S12AD.ADCSR.BIT.ADST == 1))
    {
        err = DAC_ERR_ADC_CONVERTING;
    }
    else if (SYSTEM.MSTPCRA.BIT.MSTPA17 == 1)
    {
        err = DAC_ERR_ADC_NOT_POWERED;
    }
    else
    {
        DA.DAADSCR.BIT.DAADST = 1;      // set sync with ADC
    }
#elif defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N)
    if (p_cfg->sync_with_adc == false)
    {
        DA.DAADSCR.BIT.DAADST = 0;      // do not sync with ADC
    }
    else
    {
        /*
         * This function returns an error because unit0 can not be selected
         * when synchronous conversion is enabled.
         */
        if (p_cfg->sync_unit == 0)
        {
            if ((SYSTEM.MSTPCRA.BIT.MSTPA17 == 0) && (S12AD.ADCSR.BIT.ADST == 1))
            {
                err = DAC_ERR_ADC_CONVERTING;
            }
            else if (SYSTEM.MSTPCRA.BIT.MSTPA17 == 1)
            {
                err = DAC_ERR_ADC_NOT_POWERED;
            }
            else
            {
                err = DAC_ERR_INVALID_ARG;
            }
        }
        else if ((SYSTEM.MSTPCRA.BIT.MSTPA16 == 0) && (S12AD1.ADCSR.BIT.ADST == 1))
        {
            err = DAC_ERR_ADC_CONVERTING;
        }
        else if (SYSTEM.MSTPCRA.BIT.MSTPA16 == 1)
        {
            err = DAC_ERR_ADC_NOT_POWERED;
        }
        else
        {
            /* Do Nothing */
        }

        if (err == DAC_SUCCESS)
        {
            DA.DAADUSR.BIT.AMADSEL1 = p_cfg->sync_unit;
            DA.DAADSCR.BIT.DAADST = 1;      // set sync with ADC
        }
    }
#elif defined(BSP_MCU_RX24U)
    if (p_cfg->sync_with_adc == false)
    {
        DA.DAADSCR.BIT.DAADST = 0;      // do not sync with ADC
    }
    else if ((SYSTEM.MSTPCRA.BIT.MSTPA23 == 0) && (S12AD2.ADCSR.BIT.ADST == 1))
    {
        err = DAC_ERR_ADC_CONVERTING;
    }
    else if (SYSTEM.MSTPCRA.BIT.MSTPA23 == 1)
    {
        err = DAC_ERR_ADC_NOT_POWERED;
    }
    else
    {
        DA.DAADSCR.BIT.DAADST = 1;      // set sync with ADC
    }
#endif


    /* OPTION: TURN CHANNEL CONVERTER OFF WHEN CHANNEL OUTPUT IS DISABLED */
#if defined(BSP_MCU_RX21_ALL) || defined(BSP_MCU_RX63_ALL) || defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N)
    DA.DACR.BIT.DAE = (uint8_t)((p_cfg->ch_conv_off_when_output_off == true) ? 0 : 1);
#endif

    return err;
}


/*****************************************************************************
* Function Name: power_on
* Description  : This function provides enables power to the DAC peripheral.
* Arguments    : none
* Return Value : none
******************************************************************************/
static void power_on(void)
{

    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_LPC_CGC_SWR);  // unlock
    MSTP(DA) = 0;                                               // poweron DAC
    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_LPC_CGC_SWR);   // re-lock

    return;
}


/*****************************************************************************
* Function Name: power_off
* Description  : This function removes power to the DAC peripheral
* Arguments    : none
* Return Value : none
******************************************************************************/
static void power_off(void)
{

    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_LPC_CGC_SWR);  // unlock
    MSTP(DA) = 1;                                               // poweroff DAC
    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_LPC_CGC_SWR);   // re-lock

    return;
}


/*****************************************************************************
* Function Name: R_DAC_Write
* Description  : Puts data in DAC data register for conversion on a DAC channel.
* Arguments    : chan -
*                    channel referred to
*                data -
*                    data to be written to register
* Return Value : DAC_SUCCESS -
*                    data written to register
*                DAC_ERR_BAD_CHAN -
*                    channel number invalid for part
******************************************************************************/
dac_err_t R_DAC_Write(uint8_t const chan, uint16_t data)
{
dac_err_t   err=DAC_SUCCESS;


#if (DAC_CFG_PARAM_CHECKING_ENABLE == 1)
    if (chan >= DAC_NUM_CH)
    {
        return DAC_ERR_BAD_CHAN;
    }
#endif

#if (DAC_CFG_NUM_CH == 1)
    DA.DADR0 = data;
#else
    if (chan == DAC_CH0)
    {
        DA.DADR0 = data;
    }
    else
    {
        DA.DADR1 = data;
    }
#endif

    return err;
}


/*****************************************************************************
* Function Name: R_DAC_Control
* Description  : This function is used for special hardware configuration and
*                operational control.
* Arguments    : chan -
*                    channel(s) to operate on
*                cmd -
*                    command to run
* Return Value : DAC_SUCCESS -
*                    Command completed successfully.
*                DAC_ERR_INVALID_CMD -
*                    Unknown command
*                DAC_ERR_BAD_CHAN -
*                    channel number invalid
******************************************************************************/
dac_err_t R_DAC_Control(uint8_t const chan, dac_cmd_t const cmd)
{
dac_err_t   err=DAC_SUCCESS;


#if (DAC_CFG_PARAM_CHECKING_ENABLE == 1)
    if (chan >= DAC_NUM_CH)
    {
        return DAC_ERR_BAD_CHAN;
    }
#endif

    switch (cmd)
    {
    case (DAC_CMD_OUTPUT_ON):
#if (DAC_CFG_NUM_CH == 1)
        DA.DACR.BIT.DAOE0 = 1;
#else
        if (chan == DAC_CH0)
        {
            DA.DACR.BIT.DAOE0 = 1;
        }
        else
        {
            DA.DACR.BIT.DAOE1 = 1;
        }
#endif
    break;

    case (DAC_CMD_OUTPUT_OFF):
#if (DAC_CFG_NUM_CH == 1)
        DA.DACR.BIT.DAOE0 = 0;
#else
        if (chan == DAC_CH0)
        {
            DA.DACR.BIT.DAOE0 = 0;
        }
        else
        {
            DA.DACR.BIT.DAOE1 = 0;
        }
#endif
    break;

#if defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M)
    case (DAC_CMD_AMP_ON):
        if (chan == DAC_CH0)
        {
            DA.DAAMPCR.BIT.DAAMP0 = 1;
        }
        else
        {
            DA.DAAMPCR.BIT.DAAMP1 = 1;
        }
    break;

    case (DAC_CMD_AMP_OFF):
        if (chan == DAC_CH0)
        {
            DA.DAAMPCR.BIT.DAAMP0 = 0;
        }
        else
        {
            DA.DAAMPCR.BIT.DAAMP1 = 0;
        }
    break;
#endif

    default:
        err = DAC_ERR_INVALID_CMD;
    break;
    }

    return err;
}


/*****************************************************************************
* Function Name: R_DAC_Close
* Description  : Shuts down the DAC peripheral
*
* Arguments    : none
* Return Value : DAC_SUCCESS -
*                    Peripheral disabled
******************************************************************************/
dac_err_t R_DAC_Close(void)
{

    
    DA.DACR.BYTE = DAC_CHANS_OFF;       // stop conversion/disable channels
    DA.DADR0 = 0;                       // ch0 data = 0
#if (DAC_CFG_NUM_CH > 1)
    DA.DADR1 = 0;                       // ch1 data = 0;
#endif

    DA.DADPR.BIT.DPSEL = 0;             // right justified
#if defined(BSP_MCU_RX113) || defined(BSP_MCU_RX130) || defined(BSP_MCU_RX231) || defined(BSP_MCU_RX230) \
 || defined(BSP_MCU_RX63_ALL) || defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N) \
 || defined(BSP_MCU_RX24U)
    DA.DAADSCR.BIT.DAADST = 0;          // do not sync with ADC
#endif

#if defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M)
    DA.DAAMPCR.BIT.DAAMP0 = 0;          // not used AMP0
    DA.DAAMPCR.BIT.DAAMP1 = 0;          // not used AMP1
#endif

#if defined(BSP_MCU_RX64_ALL) || defined(BSP_MCU_RX71M) || defined(BSP_MCU_RX65N)
    DA.DAADUSR.BIT.AMADSEL1 = 0;        // not sync unit1
#endif

    power_off();

    R_BSP_HardwareUnlock(BSP_LOCK_DA);

    return DAC_SUCCESS;
}


/*****************************************************************************
* Function Name: R_DAC_GetVersion
* Description  : Returns the version of this module. The version number is 
*                encoded such that the top two bytes are the major version
*                number and the bottom two bytes are the minor version number.
* Arguments    : none
* Return Value : version number
******************************************************************************/
#pragma inline(R_DAC_GetVersion)
uint32_t  R_DAC_GetVersion(void)
{
uint32_t const version = (DAC_VERSION_MAJOR << 16) | DAC_VERSION_MINOR;

    return version;
}
